#include <unistd.h>
#include <string.h>
#include <io.h>
#include <stdio.h>
#include "system.h"
#include "ADXL345.h"


#include "terasic_includes.h"
#include "I2C.h"
alt_u8 int_src[8] = {0};//at the beginning all interrupt 0

bool Gsensor_init(void)
{
    if(ADXL345_init())
        return TRUE;
    else 
        return FALSE;
    
}

bool Gsensor_uinit(void)
{
    if(standby_mode())
        return TRUE;
    else
        return FALSE;
        
}


bool Gsensor_AutoCalibration(void)
{
     alt_16 x,y,z;
     alt_8 x_set,y_set,z_set;
     
     x = y = z = 0;
     x_set = y_set = z_set = 0;
     while(Gsensor_getXYZ(&x,&y,&z)==TRUE)
     {
        if(x && y && z)
        {
            if(x>0) 
                x_set--; 
            else if(x<0)
                x_set++; 
            else
                x_set = x_set;
            if(y>0) 
                y_set--; 
            else if(y<0)
                y_set++; 
            else
                y_set = y_set;

            if(z>0) 
                z_set--;
             else if(z<0)
                z_set++; 
            else
                z_set = z_set;
            
            I2C_write_Reg(OFSX,x_set);
            I2C_write_Reg(OFSY,y_set);
            I2C_write_Reg(OFSZ,z_set);
            
                
        }
        else
        {
            printf("set XYZ bias OK\n");
            return TRUE;
        }
     }
     printf("Gsensor_Calibration fail\n");
     return FALSE;
        
            
}

bool Gsensor_selftest(alt_u8 select)
{
    alt_u8 RegAddr,RegValue;
    RegValue = 0;
    RegAddr = DATA_FORMAT;
    
    if(I2C_read_Reg(RegAddr,&RegValue))
    {
        if(select)
            RegValue |= 0x80;//set ADXL345 selftest bit to make ADXL345 selftest enable
        else
            RegValue |= (~0x80);//clean ADXL345 selftest bit to make ADXL345 selftest disable
        I2C_write_Reg(RegAddr,RegValue);
        return TRUE;
    }
    return FALSE;

}

bool measure_mode(void)
{
    alt_u8 RegAddr,RegValue;
    RegValue = 0;
    RegAddr = POWER_CTL;
    
    if(I2C_read_Reg(RegAddr,&RegValue))
    {
        RegValue |= 0x08;//set measure bit to make ADXL345 enter measure Mode
        I2C_write_Reg(RegAddr,RegValue);
        return TRUE;
    }
    return FALSE;
    
}

bool standby_mode(void)
{
    alt_u8 RegAddr,RegValue;
    RegValue = 0;
    RegAddr = POWER_CTL;
    
    if(I2C_read_Reg(RegAddr,&RegValue))
    {
        RegValue &= (~0x08);//clear measure bit to make ADXL345 enter standby Mode
        I2C_write_Reg(RegAddr,RegValue);
        return TRUE;
    }
    return FALSE;
    
}
bool set_range(alt_u8 range)
{
    alt_u8 RegAddr,RegValue;
    RegValue = 0;
    RegAddr = DATA_FORMAT;
    
    if(I2C_read_Reg(RegAddr,&RegValue))
    {
        RegValue &= 0xFC;//clean the range bits--D1~D0
        switch(range)
        {
            case 0x02:  RegValue |= 0x00; break;  //2 g 
            case 0x04:  RegValue |= 0x01; break;  //4 g 
            case 0x08:  RegValue |= 0x02; break;  //8 g 
            case 0x10:  RegValue |= 0x03; break;  //16 g 
            default:    break;
        }
        I2C_write_Reg(RegAddr,RegValue);
        return TRUE;
    }
    return FALSE;
    
}

bool get_range(alt_u8 *range)
{
    alt_u8 RegAddr,RegValue;
    RegValue = 0;
    RegAddr = DATA_FORMAT;
    
    if(I2C_read_Reg(RegAddr,&RegValue))
    {
        RegValue &= 0x03;//keep the range bits--D1~D0
        switch(RegValue)
        {
            case 0x00:  *range = 0x02;break;   //2 g 
            case 0x01:  *range = 0x04;break;   //4 g 
            case 0x02:  *range = 0x08;break;   //8 g 
            case 0x03:  *range = 0x10;break;   //16 g 
            default: break;
        }
        return TRUE;
    }
    return FALSE;
    
}

bool Gsensor_interrupt_ISR(void)
{
    alt_u8 reg_value = 0;
    alt_u8 i = 0;
    if(I2C_read_Reg(INT_SOURCE,&reg_value))
    {
        for(i = 0;i<8;i++)
            int_src[i] = (reg_value & (0x01<<i))?1:0;
        if(int_src[DATA_READY]) printf("interrupt:DATA_READY!\n");
        if(int_src[SINGLE_TAP]) printf("interrupt:SINGLE_TAP!\n");
        if(int_src[DOUBLE_TAP]) printf("interrupt:DOUBLE_TAP!\n");
        if(int_src[ACTIVITY])   printf("interrupt:ACTIVITY!\n");
        if(int_src[INACTIVITY]) printf("interrupt:INACTIVITY!\n");
        if(int_src[FREE_FALL])  printf("interrupt:FREE_FALL!\n");
        if(int_src[EATERMARK])  printf("interrupt:EATERMARK!\n");
        if(int_src[OVERRUN])    printf("interrupt:OVERRUN!\n");
        return TRUE;
    }
    return FALSE;
}

void I2C_write_Reg(alt_u8 address,alt_u8 value)
{   
    
    if(!I2C_Write(ADXL345_SCL_BASE,
                  ADXL345_SDA_BASE,
                  ADXL345_WRITE_ADDR,
                  address,
                  value))
    printf("write address:%x failed\n",address);
}

bool I2C_read_Reg(alt_u8 address,alt_u8 *value)
{
    if(!I2C_Read(ADXL345_SCL_BASE,
              ADXL345_SDA_BASE,
              ADXL345_WRITE_ADDR,
              address,
              value))
    {   
        printf("read address:%x failed\n",address);
        return FALSE;
    }
    return TRUE;
    
}

bool ADXL345_init(void)
{
    alt_u8 reg_value = 0;
    if(I2C_read_Reg(DEVID,&reg_value)) //read ADXL345's ID number for check
    {
        printf("its ADXL345's ID=%x;\n", reg_value);
    }
    else
    {
        printf("ADXL345 initialize failed!\n");
        return FALSE;
    }
    usleep(10000);//10ms
//    I2C_write_Reg(THRESH_TAP,0x05); //5/16g
//
//    I2C_write_Reg(OFSX,0x05);       //78mg
//
//    I2C_write_Reg(OFSY,0x05);       //78mg 
//      
//    I2C_write_Reg(OFSZ,0x05);       //78mg    
//  
//    I2C_write_Reg(DUR,0x50);        //50ms 
//
//    I2C_write_Reg(LATENT,0x0A);     //12.5ms 
//  
//    I2C_write_Reg(WINDOW,0x14);     //25ms 
// 
//    I2C_write_Reg(THRESH_ACT,0x08); //0.5g  
//     
//    I2C_write_Reg(THRESH_INACT,0x02);//0.125g  
//    
//    I2C_write_Reg(TIME_INACT,0x1d); //30s    
//    
//    I2C_write_Reg(ACT_INACT_CTL,0x7F);//DC compule mode,enable  all x-y-z detecting activity or inactivity
//    
//    I2C_write_Reg(THRESH_FF,0x02);  //0.125g   
//      
//    I2C_write_Reg(TIME_FF,0x14);     //100ms   
//        
//    I2C_write_Reg(TAP_AXES,0x07);   //spress=0,and enable all AXES tap 
//         
//    I2C_write_Reg(BW_RATE,0x0D);    //normal operation and  date rate=800Hz,for the  I2C'baudrate is set as 100Khz 
//    //enter standby mode    
//    //I2C_write_Reg(POWER_CTL,0x20);  //link=1,Auto_sleep=0 to less comsume power,and standby &nomal mode         
//    I2C_write_Reg(INT_MAP,0x00);    //map all interrupt to INT1,All selected interrupts for a given pin are ORed.           
//    I2C_write_Reg(DATA_FORMAT,0x2C);//disable self test,INT_INVERT=0,and Full resultion = 0; LSB=2g ,I2C-4 wire  
//    //before enable the interrupt,must configure well the interrupt source and interrupt map
//    I2C_write_Reg(INT_ENABLE,0xFF);//enable all the interrupt
//    
//    //enter measure&nomal mode,make ADXL345 start to work 
//    I2C_write_Reg(POWER_CTL,0x28);//link=1,Auto_sleep=0 to less comsume power,and mesure &nomal mode 
    
//    I2C_write_Reg(POWER_CTL,0x08);  //Power CTL: Measure mode
//    I2C_write_Reg(BW_RATE,0x0C);  //Rate: 200Hz
//    I2C_write_Reg(DATA_FORMAT,0x01);  //Data Format: 8g right justified   128=1g
//    //I2C_WRITE_REGISTER(0x2E,0xE0);    //Int En: Data Rdy, Single Tap, Doulbe Tap
//    I2C_write_Reg(INT_ENABLE,0xE4);  //Int En: Data Rdy, Single Tap, Doulbe Tap,Free fall
//    I2C_write_Reg(TAP_AXES,0x01);  //Z Axis Tap
//    I2C_write_Reg(THRESH_TAP,0x20);  //Tap Threshold: 2G;
//    I2C_write_Reg(THRESH_FF,0x09);  //FreeFall Threshold: 300mg;
//    I2C_write_Reg(TIME_FF,0x14);  //FreeFall Timing:100ms
//    I2C_write_Reg(DUR,0x50);  //Dur:50ms
//    I2C_write_Reg(LATENT,0x20);  //Latent: 40ms
//    I2C_write_Reg(WINDOW,0xF0);  //Window: 300ms

    I2C_write_Reg(POWER_CTL,0x08);  //Power CTL: Measure mode
    I2C_write_Reg(BW_RATE,0x09);  //
    I2C_write_Reg(DATA_FORMAT,0x08);  //D
    //I2C_WRITE_REGISTER(0x2E,0xE0);    //
    I2C_write_Reg(INT_ENABLE,0xE4);  //
    I2C_write_Reg(TAP_AXES,0x01);  //
    I2C_write_Reg(THRESH_TAP,0x20);  //
    I2C_write_Reg(THRESH_FF,0x09);  //
    I2C_write_Reg(TIME_FF,0x14);  //
    I2C_write_Reg(DUR,0x50);  //
    I2C_write_Reg(LATENT,0x20);  //
    I2C_write_Reg(WINDOW,0xF0);  //
    
    return TRUE;

}

alt_u8 I2C_read_nReg(alt_u8 readaddr,alt_u8 readdata[],alt_u8 len)
{
    alt_u8 actual_read_num = 0;
    if(I2C_MultipleRead(ADXL345_SCL_BASE,
                       ADXL345_SDA_BASE,
                       ADXL345_WRITE_ADDR,
                     //readaddr,
                       readdata,
                       len))
          actual_read_num = len;           
    return actual_read_num;

}

alt_u8 read_all_ADXL3435_register(void)
{
    alt_u8 actual_read_num = 0;
    alt_u8 reg_value = 0;
    alt_u8 addr = 0;
     
    for(addr = 0x1D;addr<=0x39;addr++)
    {
        if(I2C_read_Reg(addr,&reg_value))
        {
            actual_read_num++;
            usleep(10000);//10ms
            printf("read address=%x,and its value=%x;\n",addr,reg_value);
        }
    }
    return actual_read_num;
        

}

void ADXL345_test(void)
{
    alt_u8 actual_read_num = 0;
    alt_u8 reg_value = 0;
    alt_u8 addr = DEVID;
    alt_u8 readdata[6] = {0};
    alt_u8 index = 0;

    actual_read_num = I2C_read_Reg(addr,&reg_value);
    printf("read address=%x,and its ADXL345's ID=%x;\n",addr,reg_value);
    usleep(10000);//10ms
     
    
     addr = THRESH_TAP;
     I2C_write_Reg(addr,0x55);
     printf("write address=%x,and the write value=%x;\n",addr,0x55);
     usleep(10000);//10ms
     
     actual_read_num = I2C_read_Reg(addr,&reg_value);
     printf("read address=%x,and its value=%x;\n\n",addr,reg_value);
     usleep(10000);//10ms
     
     printf("\n");
     printf("***************************************\n");
     printf("***************************************\n");
     printf("\n");
     usleep(10000);//10ms
     addr = DATAX0; 
     actual_read_num = I2C_read_nReg(DATAX0,readdata,6);
     printf("read %d Byte from ADXL345,and DATAX0=%x;DATAX1=%x;DATAY0=%x;DATAY1=%x;DATAZ0=%x;DATAZ1=%x\n",actual_read_num,readdata[index++],readdata[index++],readdata[index++],readdata[index++],readdata[index++],readdata[index++]);
     usleep(10000);//10ms
     printf("\n");
     printf("***************************************\n");
     printf("***************************************\n");
     printf("\n");
     usleep(10000);//10ms
   
}

bool Gsensor_getXYZ(alt_16 *x,alt_16 *y,alt_16 *z)
{
    bool read_OK = TRUE;
    alt_u8 reg_value[6] = {0};
    
//    alt_u8 DataX0, DataX1, DataY0, DataY1, DataZ0, DataZ1;
//    alt_16 DataX, DataY, DataZ;
//    
//    I2C_read_Reg(DATAX0,&DataX0);
//    I2C_read_Reg(DATAX1,&DataX1);
//    I2C_read_Reg(DATAY0,&DataY0);
//    I2C_read_Reg(DATAY1,&DataY1);
//    I2C_read_Reg(DATAZ0,&DataZ0);
//    I2C_read_Reg(DATAZ1,&DataZ1);
//    
//
//    DataX=DataX0;
//    DataX=(DataX<<8) | DataX1;
//
//    DataY=DataY0;
//    DataY=(DataY<<8) | DataY1;
//
//    DataZ=DataZ0;
//    DataZ=(DataZ<<8) | DataZ1;
//    
//    *x = DataX;
//    *y = DataY;
//    *z = DataZ;
                
    /*
    alt_u8 RegAddr = 0;
    RegAddr = DATAX0;
    read_OK = I2C_read_Reg(RegAddr,&reg_value[0]);
    RegAddr = DATAX1;
    read_OK = I2C_read_Reg(RegAddr,&reg_value[1]);
    RegAddr = DATAY0;
    read_OK = I2C_read_Reg(RegAddr,&reg_value[2]);
    RegAddr = DATAY1;
    read_OK = I2C_read_Reg(RegAddr,&reg_value[3]);
    RegAddr = DATAZ0;
    read_OK = I2C_read_Reg(RegAddr,&reg_value[4]);
    RegAddr = DATAZ1;
    read_OK = I2C_read_Reg(RegAddr,&reg_value[5]);
    */
    if(I2C_read_nReg(DATAX0,reg_value,6))
        read_OK = TRUE;
    
    *x = (reg_value[1]<<8) | reg_value[0];
    *y = (reg_value[3]<<8) | reg_value[2];
    *z = (reg_value[5]<<8) | reg_value[4];
    
    return read_OK;
}

/*
 * for debug
 */
 void RegPrintf(alt_u8 RegAddr)
 {
        alt_u8 reg_value = 0;
        if(I2C_read_Reg(RegAddr,&reg_value))
            printf("read address=%x,and its value=%d;\n",RegAddr,reg_value);
 }
